@@ -94,7 +94,7 @@ rb_psp_source_new (RBShell *shell, GnomeVFSVolume *volume)
 					  "entry-type", entry_type,
 					  "volume", volume,
 					  "shell", shell,
-					  "sourcelist-group", RB_SOURCELIST_GROUP_REMOVABLE,
+					  "source-group", RB_SOURCE_GROUP_DEVICES,
 					  NULL));
 
 	rb_shell_register_entry_type_for_source (shell, RB_SOURCE (source), entry_type);
@@ -189,7 +189,7 @@ rb_psp_source_create_playlists (RBGenericPlayerSource *source)
 	g_free (mount_path);
 }
 
-#ifdef HAVE_HAL_0_5
+#ifdef HAVE_HAL
 
 static gboolean
 hal_udi_is_psp (const char *udi)
@@ -200,6 +200,7 @@ hal_udi_is_psp (const char *udi)
 	char *parent_name;
 	gboolean result;
 	DBusError error;
+	gboolean inited = FALSE;
 
 	result = FALSE;
 	dbus_error_init (&error);
@@ -221,6 +222,7 @@ hal_udi_is_psp (const char *udi)
 	if (!libhal_ctx_init (ctx, &error) || dbus_error_is_set (&error))
 		goto end;
 
+	inited = TRUE;
 	parent_udi = libhal_device_get_property_string (ctx, udi,
 			"info.parent", &error);
 	if (parent_udi == NULL || dbus_error_is_set (&error))
@@ -246,7 +248,8 @@ end:
 	}
 
 	if (ctx) {
-		libhal_ctx_shutdown (ctx, &error);
+		if (inited)
+			libhal_ctx_shutdown (ctx, &error);
 		libhal_ctx_free(ctx);
 	}
 
@@ -254,38 +257,6 @@ end:
 
 	return result;
 }
-
-#elif HAVE_HAL_0_2
-
-static gboolean
-hal_udi_is_psp (const char *udi)
-{
-	LibHalContext *ctx;
-	char *parent_udi;
-	char *parent_name;
-	gboolean result;
-
-	result = FALSE;
-	ctx = hal_initialize (NULL, FALSE);
-	if (ctx == NULL) {
-		return FALSE;
-	}
-	parent_udi = hal_device_get_property_string (ctx, udi,
-						     "info.parent");
-	parent_name = hal_device_get_property_string (ctx, parent_udi,
-						      "storage.model");
-	g_free (parent_udi);
-
-	if (parent_name != NULL && strcmp (parent_name, "PSP") == 0) {
-		result = TRUE;
-	}
-
-	g_free (parent_name);
-	hal_shutdown (ctx);
-
-	return result;
-}
-
 #endif
 
 gboolean
