@@ -29,6 +29,7 @@
 
 #include "config.h"
 #include "rb-refstring.h"
+#include "rb-string-value-map.h"
 #include "rhythmdb-query-results.h"
 
 G_BEGIN_DECLS
@@ -182,6 +183,7 @@ typedef enum
 	/* synthetic properties */
 	RHYTHMDB_PROP_SEARCH_MATCH,
 	RHYTHMDB_PROP_YEAR,
+	RHYTHMDB_PROP_KEYWORD, /**/
 
 	/* Podcast properties */
 	RHYTHMDB_PROP_STATUS,
@@ -194,6 +196,10 @@ typedef enum
 	RHYTHMDB_PROP_POST_TIME,
 
 	RHYTHMDB_PROP_MUSICBRAINZ_TRACKID,
+	RHYTHMDB_PROP_MUSICBRAINZ_ARTISTID,
+	RHYTHMDB_PROP_MUSICBRAINZ_ALBUMID,
+	RHYTHMDB_PROP_MUSICBRAINZ_ALBUMARTISTID,
+	RHYTHMDB_PROP_MUSICBRAINZ_ARTISTSORTNAME,
 
 	RHYTHMDB_NUM_PROPERTIES
 } RhythmDBPropType;
@@ -266,8 +272,10 @@ typedef struct
 	void	(*entry_added)		(RhythmDB *db, RhythmDBEntry *entry);
 	void	(*entry_changed)	(RhythmDB *db, RhythmDBEntry *entry, GSList *changes); /* list of RhythmDBEntryChanges */
 	void	(*entry_deleted)	(RhythmDB *db, RhythmDBEntry *entry);
+	void	(*entry_keyword_added)	(RhythmDB *db, RhythmDBEntry *entry, RBRefString *keyword);
+	void	(*entry_keyword_removed)(RhythmDB *db, RhythmDBEntry *entry, RBRefString *keyword);
 	GValue *(*entry_extra_metadata_request) (RhythmDB *db, RhythmDBEntry *entry);
-	void    (*entry_extra_metadata_gather) (RhythmDB *db, RhythmDBEntry *entry, GHashTable *data);
+	void    (*entry_extra_metadata_gather) (RhythmDB *db, RhythmDBEntry *entry, RBStringValueMap *data);
 	void	(*entry_extra_metadata_notify) (RhythmDB *db, RhythmDBEntry *entry, const char *field, GValue *metadata);
 	void	(*load_complete)	(RhythmDB *db);
 	void	(*save_complete)	(RhythmDB *db);
@@ -313,6 +321,18 @@ typedef struct
 	void		(*impl_entry_type_registered) (RhythmDB *db,
 						       const char *name,
 						       RhythmDBEntryType type);
+
+	gboolean	(*impl_entry_keyword_add)	(RhythmDB *db,
+							 RhythmDBEntry *entry,
+							 RBRefString *keyword);
+	gboolean	(*impl_entry_keyword_remove)	(RhythmDB *db,
+							 RhythmDBEntry *entry,
+							 RBRefString *keyword);
+	gboolean	(*impl_entry_keyword_has)	(RhythmDB *db,
+							 RhythmDBEntry *entry,
+							 RBRefString *keyword);
+	GList*		(*impl_entry_keywords_get)	(RhythmDB *db,
+							 RhythmDBEntry *entry);
 } RhythmDBClass;
 
 GType		rhythmdb_get_type	(void);
@@ -374,6 +394,18 @@ void		rhythmdb_entry_foreach_by_type  (RhythmDB *db,
 gint64		rhythmdb_entry_count_by_type	(RhythmDB *db,
 						 RhythmDBEntryType entry_type);
 
+gboolean	rhythmdb_entry_keyword_add	(RhythmDB *db,
+						 RhythmDBEntry *entry,
+						 RBRefString *keyword);
+gboolean	rhythmdb_entry_keyword_remove	(RhythmDB *db,
+						 RhythmDBEntry *entry,
+						 RBRefString *keyword);
+gboolean	rhythmdb_entry_keyword_has	(RhythmDB *db,
+						 RhythmDBEntry *entry,
+						 RBRefString *keyword);
+GList* /*<RBRefString>*/ rhythmdb_entry_keywords_get	(RhythmDB *db,
+							 RhythmDBEntry *entry);
+
 /**
  * Returns a freshly allocated GtkTreeModel which represents the query.
  * The extended arguments alternate between RhythmDBQueryType args
@@ -431,7 +463,7 @@ void		rhythmdb_emit_entry_added		(RhythmDB *db, RhythmDBEntry *entry);
 void		rhythmdb_emit_entry_deleted		(RhythmDB *db, RhythmDBEntry *entry);
 
 GValue *	rhythmdb_entry_request_extra_metadata	(RhythmDB *db, RhythmDBEntry *entry, const gchar *property_name);
-GHashTable *	rhythmdb_entry_gather_metadata		(RhythmDB *db, RhythmDBEntry *entry);
+RBStringValueMap* rhythmdb_entry_gather_metadata	(RhythmDB *db, RhythmDBEntry *entry);
 void		rhythmdb_emit_entry_extra_metadata_notify (RhythmDB *db, RhythmDBEntry *entry, const gchar *property_name, const GValue *metadata);
 
 gboolean	rhythmdb_is_busy			(RhythmDB *db);
